<?php
declare (strict_types = 1);

namespace app\admin\controller;
use think\Request;
use think\facade\View;
use think\facade\Db;

class Content extends Common
{
   
    /**
     * 显示资源列表
     *
     * @return \think\Response
     */
    public function index($id)
    {
        // 获取模型信息
        $model = Db::name('model')->find($id);
        
        // 获取分类信息
        $lst = Db::name('category')->where('model', $id)->order('id asc')->select();
        $lst = tree($lst);

        // 如果分类信息为空，再次查询分类信息
        if (empty($lst)) {
            $lst = Db::name('category')->where('model', $id)->order('id asc')->select();
        }

        // 分配数据到视图
        View::assign([
            'model' => $model,
            'lst' => json_encode($lst, JSON_UNESCAPED_UNICODE),
        ]);

        // 返回视图
        return View::fetch('index');
    }
    
   
    /**
     * 显示创建资源表单页.
     *
     * @return \think\Response
     */

    public function create()
    {
        // 获取模型参数
        $model = input('model');

        // 获取分类信息
        $lst = Db::name('category')->where('model', $model)->order('id asc')->select();
        $lst = tree($lst);

        // 如果分类信息为空，再次查询分类信息
        if (empty($lst)) {
            $lst = Db::name('category')->where('model', $model)->order('id asc')->select();
        }

        // 获取扩展字段信息
        $extfield = Db::name('extfield')->where('model', $model)->order('sort asc, id asc')->select();

        // 分配数据到视图
        View::assign([
            'lst' => json_encode($lst, JSON_UNESCAPED_UNICODE),
            'model' => $model,
            'extfield' => $extfield,
        ]);

        // 返回视图
        return View::fetch('add');
    }


    public function getcontent(Request $request)
    {
        // 获取分页限制
        $limit = $request->param('limit') ? $request->param('limit') : 10;

        // 初始化查询构建器
        $builder = Db::name('content');

        // 处理搜索参数
        if ($request->param('searchParams')) {
            $searchParams = json_decode($request->param('searchParams'));

            // 按标题搜索
            if (!empty($searchParams->title)) {
                $builder->where('title', 'like', '%' . $searchParams->title . '%');
            }
            
            // 按模型搜索
            if (!empty($searchParams->model)) {
                $builder->where('model', $searchParams->model);
            }

            // 处理分类搜索（包括子分类）
            if (!empty($searchParams->scode)) {
                // 获取分类信息
                $cate = Db::name('category')->where(['id' => $searchParams->scode])->find();

                if ($cate) {
                    // 获取所有子分类ID
                    $categoryIds = $this->getAllChildCategoryIds($cate['id']);
                    // 添加当前分类ID
                    $categoryIds[] = $cate['id'];
                    
                    // 使用分类ID数组搜索
                    if (!empty($categoryIds)) {
                        $builder->where('scode', 'in', $categoryIds);
                    }
                }
            }
        } else {
            // 无搜索参数时，按模型搜索
            $builder->where(['model' => $request->param('model')]);
        }

        // 获取分页数据
        $table = $builder->order('sort asc, id desc')
                        ->paginate(['list_rows' => $limit])
                        ->toArray();

        // 返回结果
        return json(['code' => 0, 'count' => $table['total'], 'data' => $table['data']]);
    }

    /**
     * 递归获取所有子分类ID
     *
     * @param int $categoryId 当前分类ID
     * @return array 所有子分类ID数组
     */
    private function getAllChildCategoryIds($categoryId)
    {
        $childIds = [];
        
        // 获取直接子分类
        $children = Db::name('category')->where('pid', $categoryId)->select()->toArray();
        
        foreach ($children as $child) {
            // 添加子分类ID
            $childIds[] = $child['id'];
            // 递归获取子分类的子分类
            $grandChildIds = $this->getAllChildCategoryIds($child['id']);
            if (!empty($grandChildIds)) {
                $childIds = array_merge($childIds, $grandChildIds);
            }
        }
        
        return $childIds;
    }
    

    /**
     * 保存新建的资源
     *
     * @param  \think\Request  $request
     * @return \think\Response
     */
    public function save(Request $request)
    {
        // 接收提交的数据
        $data = input('post.');

        // 获取分类信息
        $cate = Db::name('category')->where(['id' => $data['scode']])->find();
        $data['name'] = $cate['name'];
        $data['model'] = $cate['model'];

        if (empty($data['description']) && isset($data['content']) && is_string($data['content'])) {
            // 1. 移除HTML标签
            $text = strip_tags($data['content']);
            // 2. 替换特殊空白字符为普通空格
            $text = str_replace(['&nbsp;', '　'], ' ', $text);
            // 3. 将多个连续空格替换为单个空格
            $text = preg_replace('/\s+/', ' ', $text);
            // 4. 去除首尾空白
            $text = trim($text);
            // 5. 截取适当长度
            $data['description'] = mb_substr($text, 0, 300, 'utf-8');
        }

        //判断URL名称是否存在
        if($data['filename']!==''){
            // 检查 URL 名称是否合法（不允许包含斜杠）
            if (!preg_match('/^[a-zA-Z0-9\-_]+$/', $data['filename'])) {
                return json(['msg' => 'URL名称只允许字母、数字、横线、下划线组成，不允许包含斜杠', 'status' => 'fail']);
            }
            $filename = Db::name('content')->where(['filename' => $data['filename']])->find();
            if ($filename) {
                return json(['msg' => 'URL名称已存在，请重新设置', 'status' => 'fail']);
            }
        }

        // 如果有外链，检查是否包含协议前缀
        if (!empty($data['outlink'])) {
            // 检查是否包含协议前缀 http:// 或 https://
            if (!preg_match('/^https?:\/\//i', $data['outlink'])) {
                // 如果不包含协议前缀，添加 http://
                $data['outlink'] = 'http://' . $data['outlink'];
            }
        }

        

        // 无缩略图时，自动提取内容第一张图为缩略图, 没有内容图的话，设置缩略图为指定空图片
        if (empty($data['ico'])) {
            preg_match_all('/<img.+?src="(?P<src>[^"]+)"/i', $data['content'], $matches);
            $images = $matches['src'];
            $data['ico'] = $images[0] ?? '/public/static/admin/images/nopic.png';
        }

        // 处理自定义字段
        $prefix = 'ext_';
        $extFields = array_filter($data, function($key) use ($prefix) {
            return strpos($key, $prefix) === 0;
        }, ARRAY_FILTER_USE_KEY);

        // 插入内容数据
        $contentId = Db::name('content')->strict(false)->insertGetId($data);

        
        // 如果设置了url名称，那就链接后缀以filename为准
        $contentlink = $contentId;
        if($data['filename'] !== '') {
            $contentlink = $data['filename'];
        }

        // 生成链接地址 - 简化为直接使用分类的link
        // 使用分类的link作为基础，去掉末尾的斜杠
        $baseLink = rtrim($cate['link'], '/');
        // 生成内容链接
        $link = $baseLink . '/' . $contentlink . '.html';
        
        // 确保链接格式正确，替换多余斜杠
        $link = preg_replace('#/+#', '/', $link);
        
        Db::name('content')->where(['id' => $contentId])->update(['link' => $link]);

        // 插入扩展字段数据
        foreach ($extFields as $key => $value) {
            $extField = Db::name('extfield')->where('field', $key)->find();
            Db::name('content_ext')->insert([
                'content' => $contentId,
                'field' => $key,
                'value' => $value,
                'type' => $extField['type'],
                'model' => $extField['model'],
                'description' => $extField['description']
            ]);
        }

        // 返回操作结果
        return $contentId ? json(['msg' => '添加成功', 'status' => 'success']) : json(['msg' => '添加失败', 'status' => 'fail']);
    }

    public function edit($id)
    {
        // 获取内容信息
        $edit = Db::name('content')->find($id);
        $pics = explode(',', $edit['pics']); // 字符串转换为数组

        // 获取扩展字段信息
        $content_ext = Db::name('content_ext')->where(['content' => $edit['id'], 'model' => $edit['model']])->order('id asc')->select();

        // 获取分类信息
        $lst = Db::name('category')->where(['model' => $edit['model']])->order('id asc')->select();
        $lst = tree($lst);
        if (empty($lst)) {
            $lst = Db::name('category')->where(['model' => $edit['model']])->order('id asc')->select();
        }

        // 分配数据到视图
        View::assign([
            'edit' => $edit,
            'pics' => $pics,
            'lst' => json_encode($lst, JSON_UNESCAPED_UNICODE),
            'content_ext' => $content_ext,
        ]);

        return View::fetch('edit');
    }


    /**
     * 保存更新的资源
     *
     * @param  \think\Request  $request
     * @param  int  $id
     * @return \think\Response
     */
    public function update(Request $request, $id)
    {
        // 接收提交的数据
        $data = input('post.');

        // 获取分类信息
        $cate = Db::name('category')->where(['id' => $data['scode']])->find();
        $data['name'] = $cate['name'];
        $data['model'] = $cate['model'];

        if (empty($data['description']) && isset($data['content']) && is_string($data['content'])) {
            // 1. 移除HTML标签
            $text = strip_tags($data['content']);
            // 2. 替换特殊空白字符为普通空格
            $text = str_replace(['&nbsp;', '　'], ' ', $text);
            // 3. 将多个连续空格替换为单个空格
            $text = preg_replace('/\s+/', ' ', $text);
            // 4. 去除首尾空白
            $text = trim($text);
            // 5. 截取适当长度
            $data['description'] = mb_substr($text, 0, 300, 'utf-8');
        }

       
        // 获取数据库中的原始数据
        $originalData = Db::name('content')->where('id', $id)->find();

        // 获取 content_ext 表中的扩展字段数据
        $extFields = Db::name('content_ext')->where('content', $id)->select()->toArray();

        // 将扩展字段数据合并到原始数据中
        foreach ($extFields as $extField) {
            $originalData[$extField['field']] = $extField['value'];
        }
       
        // 比较数据是否有变化
        $hasChanges = false;
        foreach ($data as $key => $value) {
            if (isset($originalData[$key]) && $originalData[$key] != $value) {
                $hasChanges = true;
                break;
            }
        }

        if (!$hasChanges) {
            return json(['status' => 'fail', 'msg' => '没有做任何修改']);
        }


        // 如果有外链，检查是否包含协议前缀
        if (!empty($data['outlink'])) {
            // 检查是否包含协议前缀 http:// 或 https://
            if (!preg_match('/^https?:\/\//i', $data['outlink'])) {
                // 如果不包含协议前缀，添加 http://
                $data['outlink'] = 'http://' . $data['outlink'];
            }
        }


        // 无缩略图时，自动提取内容第一张图为缩略图, 没有内容图的话，设置缩略图为指定空图片
        if (empty($data['ico'])) {
            preg_match_all('/<img.+?src="(?P<src>[^"]+)"/i', $data['content'], $matches);
            $images = $matches['src'];
            $data['ico'] = $images[0] ?? '/public/static/admin/images/nopic.png';
        }

        // 如果设置了url名称，那就链接后缀以filename为准
        $contentlink=$id;
        if($data['filename']!==''){
            // 检查 URL 名称是否合法（不允许包含斜杠）
            if (!preg_match('/^[a-zA-Z0-9\-_]+$/', $data['filename'])) {
                return json(['msg' => 'URL名称只允许字母、数字、横线、下划线组成，不允许包含斜杠', 'status' => 'fail']);
            }

            $contentlink=$data['filename'];
            //判断URL名称是否存在
            $filename = Db::name('content')->where('id','<>',$id)->where(['filename' => $data['filename']])->find();
            if ($filename) {
                return json(['msg' => 'URL名称已存在，请重新设置', 'status' => 'fail']);
            }
        }

       
        if ($data['filename'] == '') {
            $link = $cate['link'] . $id. '.html';
        } else {
            $link = $cate['link'] . $data['filename']. '.html';
        }

        
        $data['link'] = $link;

        // 更新内容数据
        Db::name('content')->strict(false)->where(['id' => $id])->update($data);

        // 处理自定义字段
        $prefix = 'ext_';
        $extFields = array_filter($data, function($key) use ($prefix) {
            return strpos($key, $prefix) === 0;
        }, ARRAY_FILTER_USE_KEY);

        // 更新扩展字段数据
        foreach ($extFields as $key => $value) {
            $extField = Db::name('extfield')->where('field', $key)->find();
            Db::name('content_ext')->where(['content' => $id, 'field' => $extField['field']])->update(['value' => $value]);
        }

        // 返回操作结果
        return json(['status' => 'success', 'msg' => '更新成功']);
    }


    
    public function click()
    {
        // 接收提交的数据
        $data = input('post.');

        // 更新点击量
        $result = Db::name('content')->where('id', $data['id'])->update([$data['field'] => $data['value']]);

        // 返回操作结果
        if ($result) {
            return json(['msg' => '编辑成功', 'status' => 'success']);
        } else {
            return json(['msg' => '编辑失败', 'status' => 'fail']);
        }
    }


    public function istop(Request $request)
    {
        // 获取请求参数
        $id = $request->param('id');
        $istop = $request->param('istop');

        // 切换置顶状态
        $newStatus = $istop ? 0 : 1;
        $result = Db::name('content')->where('id', $id)->update(['istop' => $newStatus]);

        // 返回操作结果
        if ($result) {
            return json(['msg' => '操作成功', 'status' => 'success']);
        } else {
            return json(['msg' => '操作失败', 'status' => 'fail']);
        }
    }

    public function isrecommend(Request $request)
    {
        // 获取请求参数
        $id = $request->param('id');
        $isrecommend = $request->param('isrecommend');

        // 切换推荐状态
        $newStatus = $isrecommend ? 0 : 1;
        $result = Db::name('content')->where('id', $id)->update(['isrecommend' => $newStatus]);

        // 返回操作结果
        if ($result) {
            return json(['msg' => '操作成功', 'status' => 'success']);
        } else {
            return json(['msg' => '操作失败', 'status' => 'fail']);
        }
    }

    public function isheadline(Request $request)
    {
        // 获取请求参数
        $id = $request->param('id');
        $isheadline = $request->param('isheadline');

        // 切换头条状态
        $newStatus = $isheadline ? 0 : 1;
        $result = Db::name('content')->where('id', $id)->update(['isheadline' => $newStatus]);

        // 返回操作结果
        if ($result) {
            return json(['msg' => '操作成功', 'status' => 'success']);
        } else {
            return json(['msg' => '操作失败', 'status' => 'fail']);
        }
    }

    public function status(Request $request)
    {
        // 获取请求参数
        $id = $request->param('id');
        $status = $request->param('status');

        // 切换状态
        $newStatus = $status ? 0 : 1;
        $result = Db::name('content')->where('id', $id)->update(['status' => $newStatus]);

        // 返回操作结果
        if ($result) {
            return json(['msg' => '操作成功', 'status' => 'success']);
        } else {
            return json(['msg' => '操作失败', 'status' => 'fail']);
        }
    }


    /**
     * 删除指定资源
     *
     * @param  int  $id
     * @return \think\Response
     */
    public function delete($id)
    {
        // 删除内容数据
        $res = Db::name('content')->where('id', $id)->delete();

        // 如果删除成功，删除扩展字段数据
        if ($res) {
            Db::name('content_ext')->where('content', $id)->delete();
            return json(['status' => 'success', 'msg' => '删除成功']);
        } else {
            return json(['status' => 'fail', 'msg' => '删除失败']);
        }
    }


    public function del()
    {
        // 接收提交的数据
        $data=input('post.');

        $del= Db::name('content')->where(['id'=>$data['id']])->delete();

        if($del){
        	return json(['status'=>'success','msg'=>'批量删除成功']);
        }else{
        	return json(['status'=>'fail','msg'=>'批量删除失败']);
        }
    }




   

}
