<?php

namespace app\admin\controller;

use app\BaseController;
use think\facade\Db;
use think\File;

use think\facade\Config;
use think\facade\Env;
use think\Request;


class Upload extends Common
{


    public function get_images()
    {

		header('Content-Type: application/json');
		
        // 获取分页参数
        $page = isset($_GET['page']) ? intval($_GET['page']) : 1;
        $pageSize = isset($_GET['pageSize']) ? intval($_GET['pageSize']) : 10;
        $groupId = isset($_GET['groupId']) ? intval($_GET['groupId']) : null; // 获取分组ID

        // 获取时间范围参数
        $dateRange = isset($_GET['dateRange']) ? trim($_GET['dateRange']) : '';
        $startDate = $endDate = '';
        if ($dateRange && strpos($dateRange, ' - ') !== false) {
            list($startDate, $endDate) = explode(' - ', $dateRange);
            $startDate = trim($startDate);
            $endDate = trim($endDate);
        }

        $metadataFile = './uploads/metadata.json';
        $images = [];

        $baseUploadDir = 'uploads/image/';

        if (file_exists($metadataFile)) {
            $metadata = json_decode(file_get_contents($metadataFile), true);

            if (is_array($metadata)) {
                // 处理按上传时间逆序排序
                $uploadTimes = [];
                foreach ($metadata as $filename => $info) {
                    $uploadTime = isset($info['upload_time']) ? $info['upload_time'] : '';
                    $uploadTimes[$filename] = $uploadTime;
                }
                arsort($uploadTimes); // 按上传时间逆序排序

                // 遍历所有日期目录
                $dirs = glob($baseUploadDir . '*', GLOB_ONLYDIR);

                // 构建所有图片的路径映射
                $filePathMap = [];
                foreach ($dirs as $dir) {
                    $files = glob($dir . '/*');
                    foreach ($files as $file) {
                        $basename = basename($file);
                        $filePathMap[$basename] = $dir . '/';
                    }
                }

                // 根据排序后的顺序构建图片数组
                foreach ($uploadTimes as $filename => $uploadTime) {
                    $info = $metadata[$filename];

                    // 检查是否需要按分组筛选
                    // 这里加上这一行
					$itemGroupId = isset($info['group_id']) ? intval($info['group_id']) : 0;
					if ($groupId !== null && $itemGroupId != $groupId) {
						continue;
					} // 跳过不属于当前分组的图片

                    // 时间范围过滤
                    if ($startDate && $endDate && isset($info['upload_time'])) {
                        $imgDate = substr($info['upload_time'], 0, 10); // 取日期部分
                        if ($imgDate < $startDate || $imgDate > $endDate) {
                            continue; // 不在范围内跳过
                        }
                    }

                    // 查找文件实际路径
                    $dir = isset($filePathMap[$filename]) ? $filePathMap[$filename] : null;
                    if (!$dir) continue;

                    $url = '/' . $dir . $filename;
                    $name = isset($info['name']) ? $info['name'] : $filename;

                    // 获取图片尺寸
                    $width = 0;
                    $height = 0;
                    if (file_exists($dir . $filename)) {
                        $size = getimagesize($dir . $filename);
                        if ($size) {
                            $width = $size[0];
                            $height = $size[1];
                        }
                    }

                    $images[] = [
                        'url' => $url,
                        'name' => $name,
                        'width' => $width,
                        'height' => $height,
                        'group_id' => $itemGroupId,
                        'upload_time' => $uploadTime
                    ];
                }
            }
        }

        // 计算总图片数量
        $total = count($images);

        // 分页处理
        if ($pageSize < 9999) {
            $offset = ($page - 1) * $pageSize;
            $images = array_slice($images, $offset, $pageSize);
        }

        // 返回结果
        echo json_encode([
            'success' => true,
            'data' => $images,
            'total' => $total,
            'page' => $page,
            'pageSize' => $pageSize,
            'groupId' => $groupId // 添加分组ID，便于调试
        ]);
		exit;
    }

	public function get_image_groups()
	{
		// 分组数据JSON文件路径
		$dataFile = './uploads/image_groups.json';
		$imageDbFile = './uploads/images_db.json'; // 或其他存储图片的JSON文件

		// 确保目录存在
		if (!is_dir('./uploads')) {
			mkdir('./uploads', 0755, true);
		}

		
		if (!file_exists($dataFile)) {
			// 创建默认分组
			$data = [
				'groups' => [
					[
						'id' => 0,
						'name' => '默认分组',
						'count' => 0
					]
				]
			];
			file_put_contents($dataFile, json_encode($data, JSON_PRETTY_PRINT));
		} else {
			$data = json_decode(file_get_contents($dataFile), true);
		}
		
		// 重新计算每个分组的图片数量
		$this->recountGroupImages($data);
		
		// 保存更新后的数据
		file_put_contents($dataFile, json_encode($data, JSON_PRETTY_PRINT));
		
		header('Content-Type: application/json');
		echo json_encode($data);
		exit;
		
	}
	

	public function get_image_groups_save()
	{
		$dataFile = './uploads/image_groups.json';
		$imageDbFile = './images_db.json'; // 或其他存储图片的JSON文件

		// 确保目录存在
		if (!is_dir('./uploads')) {
			mkdir('./uploads', 0755, true);
		}

		$jsonData = file_get_contents('php://input');
		$data = json_decode($jsonData, true);
		
		if ($data) {
			// 重新计算每个分组的图片数量
			$this->recountGroupImages($data);
			
			// 保存更新后的数据
			file_put_contents($dataFile, json_encode($data, JSON_PRETTY_PRINT));
			header('Content-Type: application/json');
			echo json_encode(['success' => true]);
		} else {
			header('Content-Type: application/json');
			echo json_encode(['success' => false, 'message' => '数据解析失败']);
		}
		exit;
		
	}

	public function get_image_group()
	{
		$image = isset($_GET['image']) ? $_GET['image'] : '';
		if (!$image) {
			echo json_encode(['success' => false, 'message' => '缺少图片参数']);
			exit;
		}

		// 只取文件名部分
		$imageFile = basename($image);

		// 读取 metadata.json
		$metaFile = './uploads/metadata.json';
		if (!file_exists($metaFile)) {
			echo json_encode(['success' => false, 'message' => 'metadata.json不存在']);
			exit;
		}
		$meta = json_decode(file_get_contents($metaFile), true);
		if (!$meta || !isset($meta[$imageFile])) {
			echo json_encode(['success' => true, 'groupId' => 0]);
			exit;
		}
		echo json_encode(['success' => true, 'groupId' => intval($meta[$imageFile]['group_id'])]);
		exit;
	}

	//移动分组
	public function move_images()
	{
		$metadataFile = './uploads/metadata.json';

		// 检查必要参数
		if (!isset($_POST['imagePaths']) || !isset($_POST['targetGroupId'])) {
			echo json_encode([
				'success' => false,
				'message' => '缺少必要参数'
			]);
			exit;
		}

		// 获取参数
		$imagePaths = json_decode($_POST['imagePaths'], true);
		$targetGroupId = intval($_POST['targetGroupId']);

		// 验证图片路径数组
		if (!is_array($imagePaths) || empty($imagePaths)) {
			echo json_encode([
				'success' => false,
				'message' => '无效的图片路径'
			]);
			exit;
		}

		// 检查元数据文件
		if (!file_exists($metadataFile)) {
			echo json_encode([
				'success' => false,
				'message' => '图片元数据文件不存在'
			]);
			exit;
		}

		// 读取元数据
		$metadata = json_decode(file_get_contents($metadataFile), true);
		if (!is_array($metadata)) {
			echo json_encode([
				'success' => false,
				'message' => '无法解析元数据'
			]);
			exit;
		}

		// 转换图片路径为文件名
		$movedCount = 0;
		foreach ($imagePaths as $path) {
			$filename = basename($path);
			
			// 跳过不存在于元数据中的文件
			if (!isset($metadata[$filename])) {
				$strippedPath = str_replace('./uploads/', '', $path);
				if (isset($metadata[$strippedPath])) {
					$filename = $strippedPath;
				} else {
					continue;
				}
			}
			
			// 更新分组ID
			$metadata[$filename]['group_id'] = $targetGroupId;
			$movedCount++;
		}

		// 保存更新后的元数据
		if ($movedCount > 0) {
			file_put_contents($metadataFile, json_encode($metadata, JSON_UNESCAPED_UNICODE | JSON_PRETTY_PRINT));
			
			// 更新分组计数
			$this->updateAllGroupCounts();
			
			echo json_encode([
				'success' => true,
				'message' => "成功移动 $movedCount 张图片到指定分组"
			]);
		} else {
			echo json_encode([
				'success' => false,
				'message' => '未能移动任何图片'
			]);
		}
	}

	//删除分组，分组的图片移动到默认分组
	public function move_group_images()
	{
		// 获取POST参数
		$sourceGroupId = isset($_POST['sourceGroupId']) ? intval($_POST['sourceGroupId']) : 0;
		$targetGroupId = isset($_POST['targetGroupId']) ? intval($_POST['targetGroupId']) : 0;



		if ($sourceGroupId === $targetGroupId) {
			echo json_encode(['success' => false, 'message' => '源分组和目标分组不能相同']);
			exit;
		}

		// 查找正确的图片数据库文件
		$possibleFiles = [
			'./uploads/metadata.json',
			'./images_db.json',
			'./data/images.json',
			'./uploadimages.json',
			'./images.json',
			'./data/uploadimages.json',
			'./data/metadata.json'
		];

		$imagesDbFile = null;
		foreach ($possibleFiles as $file) {
			if (file_exists($file)) {
				$imagesDbFile = $file;
				break;
			}
		}

		if (!$imagesDbFile) {

			// 尝试查找任何可能的JSON文件作为备选
			$jsonFiles = glob('./*.json');
			$dataJsonFiles = glob('./data/*.json');
			$allJsonFiles = array_merge($jsonFiles, $dataJsonFiles);
			
			// 如果有任何JSON文件，尝试第一个
			if (!empty($allJsonFiles)) {
				$imagesDbFile = $allJsonFiles[0];
			} else {
				echo json_encode(['success' => false, 'message' => '无法找到图片数据库文件']);
				exit;
			}
		}

		// 读取图片数据库文件
		$imagesDbContent = file_get_contents($imagesDbFile);

		// 解析JSON
		$imagesDb = json_decode($imagesDbContent, true);
		if (!$imagesDb) {
			echo json_encode(['success' => false, 'message' => '图片数据库解析失败']);
			exit;
		}


		// 尝试找出图片数组的正确位置
		$imagesArray = null;
		if (isset($imagesDb['images'])) {
			$imagesArray = &$imagesDb['images'];
		} else {
			// 如果顶层就是图片数组
			$imagesArray = &$imagesDb;
		}

		// 更新分组ID，使用非严格比较(==)
		$changed = false;
		$movedCount = 0;

		// 检查是使用'groupId'还是'group_id'作为分组ID字段
		$groupIdField = null;
		$sampleImage = reset($imagesArray);

		// 尝试不同的可能字段名称
		$possibleFields = ['groupId', 'group_id', 'groupid', 'group-id', 'group'];
		foreach ($possibleFields as $field) {
			if ($sampleImage && isset($sampleImage[$field])) {
				$groupIdField = $field;
				break;
			}
		}

		// 如果没有找到已知的字段，尝试记录所有可用字段
		if (!$groupIdField && $sampleImage) {;
			// 尝试猜测可能的字段
			foreach (array_keys($sampleImage) as $field) {
				if (stripos($field, 'group') !== false || stripos($field, 'category') !== false) {
					$groupIdField = $field;
					break;
				}
			}
		}

		// 遍历所有图片
		foreach ($imagesArray as &$image) {
			// 如果找到了有效的分组字段名
			if ($groupIdField && isset($image[$groupIdField]) && $image[$groupIdField] == $sourceGroupId) {
				$oldGroupId = $image[$groupIdField];
				$image[$groupIdField] = $targetGroupId;
				$changed = true;
				$movedCount++;
			}
		}

		// 保存修改后的数据库
		if ($changed) {
			$newContent = json_encode($imagesDb, JSON_PRETTY_PRINT);
			$result = file_put_contents($imagesDbFile, $newContent);
			
			if ($result === false) {
				echo json_encode(['success' => false, 'message' => '保存图片数据库失败']);
				exit;
			}
			
		}

		// 尝试查找分组数据文件
		$possibleGroupFiles = [
			'./uploads/image_groups.json',
			'./image_groups.json',
			'./groups.json',
			'./data/groups.json'
		];

		$groupsFile = null;
		foreach ($possibleGroupFiles as $file) {
			if (file_exists($file)) {
				$groupsFile = $file;
				break;
			}
		}

		if (!$groupsFile) {
			// 默认使用第一个路径
			$groupsFile = $possibleGroupFiles[0];
		}

		// 从分组数据中删除源分组
		if (file_exists($groupsFile)) {
			$groups = json_decode(file_get_contents($groupsFile), true);
			if ($groups && isset($groups['groups'])) {
				// 在删除前记录分组
				
				// 查找源分组和目标分组
				$found = false;
				$targetFound = false;
				$targetIndex = -1;
				
				foreach ($groups['groups'] as $key => $group) {
					if ($group['id'] == $sourceGroupId) {
						unset($groups['groups'][$key]);
						$found = true;
					}
					if ($group['id'] == $targetGroupId) {
						$targetFound = true;
						$targetIndex = $key;
					}
				}
				
				
				// 重建索引
				$groups['groups'] = array_values($groups['groups']);
				
				// 更新目标分组的计数
				if ($targetFound && $movedCount > 0) {
					$oldCount = $groups['groups'][$targetIndex]['count'];
					$groups['groups'][$targetIndex]['count'] = $oldCount + $movedCount;
				}
				
				// 保存回文件
				$result = file_put_contents($groupsFile, json_encode($groups, JSON_PRETTY_PRINT));
			}
		} else {
			
			// 创建基本的分组结构
			$groups = [
				'groups' => [
					[
						'id' => 0,
						'name' => '默认分组',
						'count' => $movedCount
					]
				]
			];
			
			// 确保data目录存在
			$dir = dirname($groupsFile);
			if (!is_dir($dir)) {
				mkdir($dir, 0755, true);
			}
			
			// 保存新创建的分组文件
			$result = file_put_contents($groupsFile, json_encode($groups, JSON_PRETTY_PRINT));
		}

		// 更新所有分组的计数 - 重新扫描所有图片，确保计数准确
		$this->updateAllGroupCounts($imagesDbFile, $groupsFile, $groupIdField);

		echo json_encode([
			'success' => true, 
			'movedCount' => $movedCount,
			'message' => "成功将 $movedCount 张图片移动到默认分组"
		]);
	}
	

	public function get_directory_images()
	{
		$page = isset($_GET['page']) ? intval($_GET['page']) : 1;
		$pageSize = isset($_GET['pageSize']) ? intval($_GET['pageSize']) : 10;
		$directory = isset($_GET['directory']) ? $_GET['directory'] : 'uploads';

		// 目录安全性检查，确保只能访问uploads目录下的内容
		$baseDir = realpath('./uploads');
		$requestedDir = realpath('./' . $directory);

		// 检查请求的目录是否存在且在允许的范围内
		if (!$requestedDir || strpos($requestedDir, $baseDir) !== 0) {
			echo json_encode([
				'success' => false,
				'message' => '无效的目录路径',
				'data' => [],
				'total' => 0
			]);
			exit;
		}

		// 支持的图片类型
		$supportedExtensions = ['jpg', 'jpeg', 'png', 'gif', 'bmp', 'webp', 'svg'];

		// 获取指定目录中的所有图片
		$images = [];
		$files = scandir($requestedDir);

		foreach ($files as $file) {
			// 跳过.和..目录以及隐藏文件
			if ($file == '.' || $file == '..' || substr($file, 0, 1) === '.') {
				continue;
			}
			
			$filePath = $requestedDir . DIRECTORY_SEPARATOR . $file;
			
			// 只处理文件且是图片类型的文件
			if (is_file($filePath)) {
				$extension = strtolower(pathinfo($file, PATHINFO_EXTENSION));
				
				if (in_array($extension, $supportedExtensions)) {
					// 获取图片大小
					list($width, $height) = @getimagesize($filePath);
					
					// 相对路径
					$relativePath = str_replace('\\', '/', substr($filePath, strlen(realpath('.')) + 1));
					if (substr($relativePath, 0, 1) !== '/') {
						$relativePath = '/' . $relativePath;
					}
					
					$images[] = [
						'url' => $relativePath,
						'name' => $file,
						'width' => $width ?: 0,
						'height' => $height ?: 0,
						'size' => filesize($filePath),
						'lastModified' => filemtime($filePath)
					];
				}
			}
		}

		// 按最后修改时间倒序排序
		usort($images, function($a, $b) {
			return $b['lastModified'] - $a['lastModified'];
		});

		// 计算总数
		$total = count($images);

		// 分页
		$offset = ($page - 1) * $pageSize;
		$pageImages = array_slice($images, $offset, $pageSize);

		// 返回JSON格式的结果
		echo json_encode([
			'success' => true,
			'data' => $pageImages,
			'total' => $total
		]);
	}

	public function get_directory_structure()
	{
		// 获取请求的目录路径
		$basePath = isset($_GET['path']) ? $_GET['path'] : 'uploads';
		$basePath = realpath($basePath); // 转换为绝对路径

		// 安全检查：确保目录在允许范围内
		$allowedRoot = realpath('./uploads');
		if (!$basePath || strpos($basePath, $allowedRoot) !== 0) {
			echo json_encode([
				'success' => false,
				'message' => '无效的目录路径'
			]);
			exit;
		}

		// 递归获取目录结构，转为Layui Tree格式
		function scanDirectory($dir, $relativePath = '') {
			$result = [];
			
			if (!is_dir($dir)) {
				return $result;
			}
			
			$items = scandir($dir);
			foreach ($items as $item) {
				// 跳过.和..以及隐藏文件
				if ($item == '.' || $item == '..' || substr($item, 0, 1) === '.') {
					continue;
				}
				
				$path = $dir . DIRECTORY_SEPARATOR . $item;
				$currentPath = $relativePath ? $relativePath . '/' . $item : $item;
				
				if (is_dir($path)) {
					// 检查是否是最后一级目录（没有子目录的目录）
					$hasSubDirs = false;
					$subItems = scandir($path);
					foreach ($subItems as $subItem) {
						// 修正判断条件，确保正确检测子目录
						if ($subItem != '.' && $subItem != '..' && substr($subItem, 0, 1) !== '.' && is_dir($path . DIRECTORY_SEPARATOR . $subItem)) {
							$hasSubDirs = true;
							break;
						}
					}

					// 计算图片数
					$imgCount = countImagesInDir($path);

					// 创建目录节点
					$node = [
						'title' => $item . ($hasSubDirs ? '' : ' ('.$imgCount.')'),  // 只有最后一级目录显示图片数量
						'path' => $currentPath,
						'isLeaf' => false,  // 所有目录节点都不是叶子节点
						'spread' => false,  // 默认不展开
						'imgCount' => $imgCount // 保存图片数量
					];
					
					// 递归获取子目录
					$children = scanDirectory($path, $currentPath);
					if (!empty($children)) {
						$node['children'] = $children;
					}
					
					$result[] = $node;
				}
			}
			
			return $result;
		}

		// 计算目录中的图片文件数量
		function countImagesInDir($dir) {
			$count = 0;
			$imgExtensions = ['jpg', 'jpeg', 'png', 'gif', 'bmp', 'webp', 'svg'];
			
			if (is_dir($dir)) {
				$files = scandir($dir);
				foreach ($files as $file) {
					if ($file == '.' || $file == '..') continue;
					
					$filePath = $dir . DIRECTORY_SEPARATOR . $file;
					if (is_file($filePath)) {
						$ext = strtolower(pathinfo($file, PATHINFO_EXTENSION));
						if (in_array($ext, $imgExtensions)) {
							$count++;
						}
					}
				}
			}
			
			return $count;
		}

		try {
			// 创建根节点
			$treeData = [
				[
					'title' => basename($basePath),
					'path' => 'uploads',
					'isLeaf' => false,
					'spread' => true,
					'icon' => 'layui-icon layui-icon-folder',  // 根目录也使用文件夹图标
					'children' => scanDirectory($basePath, 'uploads')
				]
			];
			
			echo json_encode([
				'success' => true,
				'data' => $treeData
			]);
		} catch (Exception $e) {
			echo json_encode([
				'success' => false,
				'message' => $e->getMessage()
			]);
		}
		
	}

	//上传图片
	public function upload_images()
	{
		header('Content-Type: application/json');

		// 自动生成上传目录（按日期）
		$today = date('Ymd');
		$baseDir = "uploads/image";
		$uploadDir = "{$baseDir}/{$today}/"; // 上传目录
		$metadataFile = './uploads/metadata.json'; // 存储图片元数据的文件


		// 确保 uploads 目录存在
		if (!is_dir('./uploads')) {
			if (!mkdir('./uploads', 0777, true)) {
				exit(json_encode(['success' => false, 'msg' => "创建uploads目录失败"]));
			}
		}

		// 确保 metadata.json 文件存在且内容为 []
		if (!file_exists($metadataFile) || filesize($metadataFile) === 0) {
			if (file_put_contents($metadataFile, json_encode([])) === false) {
				exit(json_encode(['success' => false, 'msg' => "创建元数据文件失败: $metadataFile"]));
			}
		}

		// 确保基础目录和上传目录存在
		if (!is_dir($baseDir)) {
			if (!mkdir($baseDir, 0777, true)) {
				exit(json_encode(['success' => false, 'msg' => "创建基础目录失败: $baseDir"]));
			}
		}
		if (!is_dir($uploadDir)) {
			if (!mkdir($uploadDir, 0777, true)) {
				exit(json_encode(['success' => false, 'msg' => "创建上传目录失败: $uploadDir"]));
			}
		}


		// 获取分组ID，默认为0（默认分组）
		$groupId = isset($_POST['groupId']) ? intval($_POST['groupId']) : 0;

		$uploadedFiles = [];
		$errorFiles = [];

		// 读取原有元数据，确保为数组（即使文件不存在或内容为空也不会报错）
		$metadata = [];
		if (file_exists($metadataFile)) {
			$metadataContent = file_get_contents($metadataFile);
			$metadata = json_decode($metadataContent, true);
			if (!is_array($metadata)) {
				$metadata = [];
			}
		}

		if (isset($_FILES['file']['name'])) {
			if (is_array($_FILES['file']['name'])) {
				foreach ($_FILES['file']['name'] as $key => $name) {
					$tmpName = $_FILES['file']['tmp_name'][$key];
					$error = $_FILES['file']['error'][$key];
					$size = $_FILES['file']['size'][$key];


					if ($error === UPLOAD_ERR_OK) {
						$extension = strtolower(pathinfo($name, PATHINFO_EXTENSION));
						// 文件名：秒级时间戳_6位随机数.后缀
						$timestamp = time();
						$rand = mt_rand(100000, 999999);
						$uniqueName = $timestamp . $rand . '.' . $extension;
						$filePath = $uploadDir . $uniqueName;

						if (move_uploaded_file($tmpName, $filePath)) {
							// 获取微秒级时间
							$microtime = microtime(true);
							$precise_time = date('Y-m-d H:i:s', floor($microtime)) . '.' . sprintf('%03d', round(($microtime - floor($microtime)) * 1000));
							
							// 保存元数据包括分组ID
							$metadata[$uniqueName] = [
								'name' => $name,
								'upload_time' => $precise_time,
								'group_id' => $groupId
							];
							$uploadedFiles[] = [
								'url' => '/'.$filePath,
								'name' => $name,
								'group_id' => $groupId
							];
						} else {
							$errorMsg = "移动文件失败: " . (error_get_last()['message'] ?? '未知错误');
							$errorFiles[] = ['name' => $name, 'error' => $errorMsg];
						}
					} else {
						$errorMsg = "上传错误码: $error";
						$errorFiles[] = ['name' => $name, 'error' => $errorMsg];
					}
				}
			} else {
				// 单文件上传处理
				$name = $_FILES['file']['name'];
				$tmpName = $_FILES['file']['tmp_name'];
				$error = $_FILES['file']['error'];
				$size = $_FILES['file']['size'];

				if ($error === UPLOAD_ERR_OK) {
					$extension = strtolower(pathinfo($name, PATHINFO_EXTENSION));
					$timestamp = time();
					$rand = mt_rand(100000, 999999);
					$uniqueName = $timestamp . $rand . '.' . $extension;
					$filePath = $uploadDir . $uniqueName;

					if (move_uploaded_file($tmpName, $filePath)) {
						$microtime = microtime(true);
						$precise_time = date('Y-m-d H:i:s', floor($microtime)) . '.' . sprintf('%03d', round(($microtime - floor($microtime)) * 1000));
						
						// 保存元数据包括分组ID
						$metadata[$uniqueName] = [
							'name' => $name,
							'upload_time' => $precise_time,
							'group_id' => $groupId
						];
						$uploadedFiles[] = [
							'url' => '/'.$filePath,
							'name' => $name,
							'group_id' => $groupId
						];
					} else {
						$errorMsg = "移动文件失败: " . (error_get_last()['message'] ?? '未知错误');
						$errorFiles[] = ['name' => $name, 'error' => $errorMsg];
					}
				} else {
					$errorMsg = "上传错误码: $error";
					$errorFiles[] = ['name' => $name, 'error' => $errorMsg];
				}
			}

			// 批量上传时安全写入metadata.json，防止并发丢失
			$fp = fopen($metadataFile, 'c+');
			if ($fp) {
				if (flock($fp, LOCK_EX)) {
					rewind($fp); // 关键：确保从头读取
					$oldContent = stream_get_contents($fp);
					$oldMeta = [];
					if ($oldContent && trim($oldContent) !== '') {
						$oldMeta = json_decode($oldContent, true);
						if (!is_array($oldMeta)) $oldMeta = [];
					}
					// 合并新旧数据
					$metadata = array_merge($oldMeta, $metadata);
					// 回到文件头
					ftruncate($fp, 0);
					rewind($fp);
					fwrite($fp, json_encode($metadata, JSON_UNESCAPED_UNICODE | JSON_PRETTY_PRINT));
					fflush($fp);
					flock($fp, LOCK_UN);
				}
				fclose($fp);
			}
		}

		// 更新分组的图片计数
		function updateGroupCount($groupId) {
			global $metadataFile;
			
			// 读取元数据
			$metadataContent = file_get_contents($metadataFile);
			$metadata = json_decode($metadataContent, true);
			
			// 计算指定分组的图片数量
			$count = 0;
			foreach ($metadata as $item) {
				if (isset($item['group_id']) && $item['group_id'] == $groupId) {
					$count++;
				}
			}
			
			// 更新分组计数
			$groupsFile = './uploads/image_groups.json';
			if (file_exists($groupsFile)) {
				$groupsData = json_decode(file_get_contents($groupsFile), true);
				
				foreach ($groupsData['groups'] as &$group) {
					if ($group['id'] == $groupId) {
						$group['count'] = $count;
						break;
					}
				}
				
				file_put_contents($groupsFile, json_encode($groupsData, JSON_UNESCAPED_UNICODE | JSON_PRETTY_PRINT));
			}
		}

		// 返回结果
		$response = [
			'success' => !empty($uploadedFiles),
			'files' => $uploadedFiles
		];

		if (!empty($errorFiles)) {
			$response['errors'] = $errorFiles;
		}

		if (!empty($uploadedFiles)) {
			$response['lastUploaded'] = reset($uploadedFiles);
		}

		echo json_encode($response);
	}

	//删除图片
	public function delete_image()
	{
		header('Content-Type: application/json');

		$imagesDir = './uploads/'; // 图片目录
		$metadataFile = './uploads/metadata.json'; // 存储图片元数据的文件
		$groupsFile = './uploads/image_groups.json'; // 分组数据文件

		// 支持批量删除
		if (isset($_POST['imagePaths'])) {
			$imagePaths = json_decode($_POST['imagePaths'], true);
			$success = true;
			$messages = [];
			foreach ($imagePaths as $imagePath) {
				if (strpos($imagePath, '/uploads/') === 0) {
					$imagePath = '.' . $imagePath;
				}
				if (!file_exists($imagePath)) {
					$success = false;
					$messages[] = "$imagePath 不存在";
					continue;
				}
				if (unlink($imagePath)) {
					// 更新元数据
					if (file_exists($metadataFile)) {
						$metadata = json_decode(file_get_contents($metadataFile), true);
						$fileName = basename($imagePath);
						if (isset($metadata[$fileName])) {
							unset($metadata[$fileName]);
							file_put_contents($metadataFile, json_encode($metadata, JSON_UNESCAPED_UNICODE | JSON_PRETTY_PRINT));
						}
					}
				} else {
					$success = false;
					$messages[] = "$imagePath 删除失败";
				}
			}
			
			// 批量删除后更新所有分组的计数
			if ($success) {
				$this->updateAllGroupCounts();
			}
			
			echo json_encode(['success' => $success, 'message' => implode('; ', $messages)]);
			exit;
		}

		// 单张图片删除（原有逻辑）
		if (!isset($_POST['imagePath'])) {
			echo json_encode(['success' => false, 'message' => '未提供图片路径']);
			exit;
		}

		$imagePath = $_POST['imagePath'];
		if (strpos($imagePath, '/uploads/') === 0) {
			$imagePath = '.' . $imagePath;
		}

		if (!file_exists($imagePath)) {
			echo json_encode(['success' => false, 'message' => '图片文件不存在']);
			exit;
		}

		if (unlink($imagePath)) {
			if (file_exists($metadataFile)) {
				$metadata = json_decode(file_get_contents($metadataFile), true);
				$fileName = basename($imagePath);
				if (isset($metadata[$fileName])) {
					unset($metadata[$fileName]);
					file_put_contents($metadataFile, json_encode($metadata, JSON_UNESCAPED_UNICODE | JSON_PRETTY_PRINT));
				}
			}
		
			
			// 单张图片删除后更新所有分组的计数
			$this->updateAllGroupCounts();
			
			echo json_encode(['success' => true, 'message' => '图片已删除']);
		} else {
			echo json_encode(['success' => false, 'message' => '无法删除图片文件']);
		}

	}

	//上传文件
	public function upload_file()
	{
		header('Content-Type: application/json');
		// 自动生成上传目录（按日期）
		$today = date('Ymd');
		$baseDir = "uploads/file";
		$uploadDir = "{$baseDir}/{$today}/";

		if (!is_dir($baseDir)) mkdir($baseDir, 0777, true);
		if (!is_dir($uploadDir)) mkdir($uploadDir, 0777, true);

		if (isset($_FILES['file']['name'])) {
			$name = $_FILES['file']['name'];
			$tmpName = $_FILES['file']['tmp_name'];
			$error = $_FILES['file']['error'];

			if ($error === UPLOAD_ERR_OK) {
				$extension = strtolower(pathinfo($name, PATHINFO_EXTENSION));
				$uniqueName = time() . mt_rand(100000, 999999) . '.' . $extension;
				$filePath = $uploadDir . $uniqueName;

				if (move_uploaded_file($tmpName, $filePath)) {
					return json(['path'=>'/' . $filePath]);
				} else {
					echo json_encode(['success' => false, 'message' => '移动文件失败']);
					return;
				}
			} else {
				echo json_encode(['success' => false, 'message' => '上传错误码: ' . $error]);
				return;
			}
		} else {
			echo json_encode(['success' => false, 'message' => '未检测到上传文件']);
			return;
		}
		

		
	}

	//远程图片下载
	public function fetch_image()
	{
		header('Content-Type: application/json');

		$url = isset($_POST['url']) ? trim($_POST['url']) : '';
		if (!$url) {
			echo json_encode(['success' => false, 'message' => '图片地址不能为空']);
			exit;
		}

		// 校验图片链接
		if (!preg_match('/^https?:\/\/.+\.(jpg|jpeg|png|gif|bmp|webp|ico)$/i', $url)) {
			echo json_encode(['success' => false, 'message' => '图片链接格式不正确']);
			exit;
		}

		// 下载图片
		$imgData = @file_get_contents($url);
		if ($imgData === false) {
			echo json_encode(['success' => false, 'message' => '图片下载失败']);
			exit;
		}

		// 生成保存目录（与上传一致）
		$today = date('Ymd');
		$uploadDir = 'uploads/image/' . $today . '/';
		if (!is_dir($uploadDir)) mkdir($uploadDir, 0777, true);




		// 生成唯一文件名
		$ext = strtolower(pathinfo(parse_url($url, PHP_URL_PATH), PATHINFO_EXTENSION));
		$timestamp = time();
		$rand = mt_rand(100000, 999999);
		$uniqueName = $timestamp . $rand . '.' . $ext;
		$filePath = $uploadDir . $uniqueName;

		// 保存图片
		if (file_put_contents($filePath, $imgData) === false) {
			echo json_encode(['success' => false, 'message' => '图片保存失败']);
			exit;
		}

		// 写入元数据
		$metadataFile = './uploads/metadata.json';
		if (!file_exists($metadataFile)) file_put_contents($metadataFile, json_encode([]));
		$metadata = json_decode(file_get_contents($metadataFile), true);
		if (!is_array($metadata)) $metadata = [];
		$metadata[$uniqueName] = [
			'name' => basename($url),
			'upload_time' => date('Y-m-d H:i:s'),
			'group_id' => 0 // 默认分组
		];
		file_put_contents($metadataFile, json_encode($metadata, JSON_UNESCAPED_UNICODE | JSON_PRETTY_PRINT));

		// 更新分组数量
		$groupsFile = './uploads/image_groups.json';
		if (file_exists($groupsFile)) {
			$groups = json_decode(file_get_contents($groupsFile), true);
			if (isset($groups['groups']) && is_array($groups['groups'])) {
				foreach ($groups['groups'] as &$group) {
					if ($group['id'] == 0) {
						$group['count'] = isset($group['count']) ? $group['count'] + 1 : 1;
						break;
					}
				}
				file_put_contents($groupsFile, json_encode($groups, JSON_UNESCAPED_UNICODE | JSON_PRETTY_PRINT));
			}
		}

		echo json_encode([
			'success' => true,
			'url' => '/' . $filePath
		]);
	}
	

	/**
	 * 重新计算每个分组包含的图片数量
	*/
	function recountGroupImages(&$data) {
		global $imageDbFile;
		
		// 查找可能的图片数据库文件
		$possibleFiles = [
			'./uploads/metadata.json',
		];
		
		$imagesDb = null;
		foreach ($possibleFiles as $file) {
			if (file_exists($file)) {
				$content = file_get_contents($file);
				$imagesDb = json_decode($content, true);
				if ($imagesDb) {
					break;
				}
			}
		}

		if ($imagesDb === null) {
			return;
		}
		
		// 重置所有分组的计数
		foreach ($data['groups'] as &$group) {
			$group['count'] = 0;
		}
		
		// 重新计算每个分组的图片数量
		$imagesArray = isset($imagesDb['images']) ? $imagesDb['images'] : $imagesDb;
		if (!is_array($imagesArray)) {
			return;
		}
		
		// 尝试不同的字段名称
		$possibleFields = ['groupId', 'group_id', 'groupid', 'group-id', 'group'];
		
		foreach ($imagesArray as $image) {
			$groupId = 0; // 默认分组ID
			
			// 查找图片所属的分组ID
			foreach ($possibleFields as $field) {
				if (isset($image[$field])) {
					$groupId = (int)$image[$field];
					break;
				}
			}
			
			// 增加对应分组的计数
			foreach ($data['groups'] as &$group) {
				if ($group['id'] == $groupId) {
					$group['count']++;
					break;
				}
			}
		}
	}

	/**
	 * 更新所有分组的图片计数
	 */
	function updateAllGroupCounts()
	{
		global $metadataFile, $groupsFile;

		// 检查必要文件是否存在
		if (!file_exists($metadataFile) || !file_exists($groupsFile)) {
			return false;
		}

		// 读取元数据和分组数据
		$metadata = json_decode(file_get_contents($metadataFile), true);
		if (!is_array($metadata)) $metadata = [];
		$groupsData = json_decode(file_get_contents($groupsFile), true) ?: ['groups' => []];

		// 初始化计数器
		$counts = [];
		foreach ($groupsData['groups'] as $group) {
			$counts[$group['id']] = 0;
		}

		// 统计每个分组的图片数量
		foreach ($metadata as $item) {
			// 兼容对象数组和以文件名为key的对象
			if (is_array($item) && isset($item['group_id'])) {
				$groupId = intval($item['group_id']);
			} else {
				$groupId = 0;
			}
			if (isset($counts[$groupId])) {
				$counts[$groupId]++;
			} else {
				$counts[$groupId] = 1;
			}
		}
		
		// 更新分组数据中的计数
		foreach ($groupsData['groups'] as &$group) {
			$groupId = $group['id'];
			$group['count'] = isset($counts[$groupId]) ? $counts[$groupId] : 0;
		}
	
		// 保存更新后的分组数据
		file_put_contents($groupsFile, json_encode($groupsData, JSON_UNESCAPED_UNICODE | JSON_PRETTY_PRINT));
		return true;
	}



}