<?php
declare (strict_types = 1);

namespace app\index\controller;

use think\Request;
use think\facade\View;
use think\facade\Db;
use think\facade\Session;

class Index extends Common
{
    
    // 空拦截器, 实现文章路由转发
    public function _empty(Request $request)
    {   
        // 获取基本参数
        $config = Db::name('config')->where('id', 1)->find();
        $url = $request->param('any', '');
        $params = $request->param();


        // 处理后台跳转
        if ($url== $config['admin_path']) {
            return '<script>window.location.href="/admin/login"</script>';
        }

        // 处理搜索页请求
        if ($url == 'search') {
            $keyword = $request->param('keyword', '');
            $this->getSearch($keyword);
            return $this->renderForDevice('search', $config);
        }
        
        // 处理特殊页面
        if ($url == 'all') {
            return $this->all();
        }

        $url='/'.$url.'/';
        // 获取当前栏目信息
        $sort = Db::name('category')->where('filename', $url)->whereOr('link', $url)->find();

        
        // 处理URL中包含多个参数的情况（通常是内容详情页或翻页）
        if (count($params) > 1) {
            // 处理带有page参数的请求（分页）
            if (isset($params['page'])) {
                $this->getList($sort);
                return $this->renderForDevice($this->cleanFilename($sort['listtpl']), $config);
            }
            
            // 处理内容详情页
            $id = $this->extractContentId($request, $url);
            if ($id) {
                $this->getContent($id);
                return $this->renderForDevice($this->cleanFilename($sort['contenttpl']), $config);
            }
        } 
        // 处理只有一个参数的情况（通常是栏目页面）
        else {
            // 处理404
            if (!$sort) {
                return View::fetch('404.html');
            }
            
            // 获取栏目模型类型
            $model = Db::name('model')->where('id', $sort['model'])->find();
            
            // 单页模型
            if ($model['type'] == '1') {
                $this->getAbout($sort);
                return $this->renderForDevice($this->cleanFilename($sort['contenttpl']), $config);
            } 
            // 列表模型
            elseif ($model['type'] == '2') {
                $this->getList($sort);
                return $this->renderForDevice($this->cleanFilename($sort['listtpl']), $config);
            }
        }
        
        // 默认返回404
        return View::fetch('404.html');
    }

    public function data(){
        $data = Db::name('system')->where('id',1)->find();
        return json($data);
    }

    // 首页
    public function index()
    {
        //站点信息
        $system=Db::name('system')->where('id',1)->find();
        $long=Db::name('system')->where('id',1)->find();
        //配置参数
        $config=Db::name('config')->where('id',1)->find();

        // SEO信息处理
        $long['title']=$system['title'];
        $long['keywords']=$system['keywords'];
        $long['description']=$system['description'];

    
        View::assign([
            'long'=>$long,
        ]);

        //判断是否有单独手机端
        if($config['open_wap']==1){
            if(request()->isMobile()){
                //判断是否有设置过手机端域名
                if($config['wap_domain']!=''){
                    // 检查域名是否包含http前缀，没有则添加
                    $wapUrl = preg_match('/^https?:\/\//i', $config['wap_domain']) ? 
                            $config['wap_domain'] : 'http://'.$config['wap_domain'];
                    header("Location: $wapUrl");
                    exit;
                }
                return View::fetch('/wap/index');
            }else{
                return View::fetch('index');
            }
        }
        return View::fetch('index');
    }

    // 单页
    public function getAbout($sort)
    {
        // 获取配置参数及顶级栏目
        $system=Db::name('system')->where('id',1)->find();
        $long=Db::name('system')->where('id',1)->find();
        $cate=$this->getTopCategory($sort['id']);

        // 获取单页内容和自定义标签
        $content = Db::name('single')->where(['scode' => $sort['id']])->find();
        $single_ext = Db::name('single_ext')->where(['single' => $sort['id']])->select()->toArray();
        $content['single_ext'] = array_column($single_ext, 'value', 'field');

        // SEO信息处理及面包屑
        $long['title']=$sort['title']?$sort['title'] : $sort['name'] . '-' . $system['title'];
        $long['keywords'] = !empty($sort['keywords']) ? $sort['keywords'] : $system['keywords'];
        $long['description'] = !empty($sort['description']) ? $sort['description'] : $system['description'];
        $long['position'] = $this->buildBreadcrumb($sort['id']);

        // 当前栏目没有栏目图，则获取顶级栏目栏目图
        $sort['pic'] = !empty($sort['pic']) ? $sort['pic'] : $cate['pic'];

        // 输出到模板
        View::assign([
            'long'=>$long,
            'cate'=>$cate,
            'sort'=>$sort,
            'content'=>$content,
        ]);
       
    }


    // 列表页
    public function getList($sort)
    {
      
        // 获取配置参数及顶级栏目
        $system=Db::name('system')->where('id',1)->find();
        $long=Db::name('system')->where('id',1)->find();
        $cate=$this->getTopCategory($sort['id']);

        // SEO信息处理及面包屑
        $long['title']=$sort['title']?$sort['title'] : $sort['name'] . '-' . $system['title'];
        $long['keywords'] = !empty($sort['keywords']) ? $sort['keywords'] : $system['keywords'];
        $long['description'] = !empty($sort['description']) ? $sort['description'] : $system['description'];
        $long['position'] = $this->buildBreadcrumb($sort['id']);

        // 当前栏目没有栏目图，则获取顶级栏目栏目图
        $sort['pic'] = !empty($sort['pic']) ? $sort['pic'] : $cate['pic'];

        // 输出到模板
        View::assign([
            'long'=>$long,
            'cate'=>$cate,
            'sort'=>$sort,
        ]);
    }


    // 详情页
    public function getContent($id)
    {
       
        // 获取详情内容和自定义标签
        
        $content = Db::name('content')->where('id',$id)->whereOr('filename',$id)->find();
        
        $content_ext = Db::name('content_ext')->where(['content'=>$content['id']])->select()->toArray();
        $content['ext'] = array_column($content_ext, 'value', 'field');
        Db::name('content')->where(['id'=>$content['id']])->inc('click', 1)->update();

        // 获取配置参数及顶级栏目
        $system=Db::name('system')->where('id',1)->find();
        $long=Db::name('system')->where('id',1)->find();
        $cate=$this->getTopCategory($content['scode']);

        // SEO信息处理及面包屑
        $long['title']=$content['title'] . '-' . $system['title'];
        $long['keywords'] = !empty($content['keywords']) ? $content['keywords'] : $system['keywords'];
        $long['description'] = !empty($content['description']) ? $content['description'] : $system['description'];
        $long['position'] = $this->buildBreadcrumb($content['scode'], $content['title']);
      
        // 当前栏目没有栏目图，则获取顶级栏目栏目图
        $sort=Db::name('category')->where('id',$content['scode'])->find();
        $sort['pic'] = !empty($sort['pic']) ? $sort['pic'] : $cate['pic'];

        //上一条及下一条
        $content_prev=Db::name('content')->where('status',1)->where('id','<',$content['id'])->where(['scode'=>$content['scode']])->order('id desc')->limit(1)->find();
        $content_next=Db::name('content')->where('status',1)->where('id','>',$content['id'])->where(['scode'=>$content['scode']])->order('id asc')->limit(1)->find();
        if($content_prev){
            $content['prev'] = '<a href="' . $content_prev['link'] . '" title="' . $content_prev['title'] . '">' . $content_prev['title'] . '</a>';
        }else{
            $content['prev'] = '<span>暂无</span>';
        }
        if($content_next){
            $content['next'] = '<a href="' . $content_next['link'] . '" title="' . $content_next['title'] . '">' . $content_next['title'] . '</a>';
        } else {
            $content['next'] = '<span>暂无</span>';
        }
        
        //判断详情权限
        if(!$content['gid']==''){
            echo '<script>window.location.href="/login/"</script>';
        }
        
        View::assign([
            'long'=>$long,
            'cate'=>$cate,
            'sort'=>$sort,
            'content'=>$content,
        ]);
    }

    // 搜索页
    public function getSearch($keyword)
    {
        //站点信息及配置参数
        $system=Db::name('system')->where('id',1)->find();
        $long=Db::name('system')->where('id',1)->find();
        $config=Db::name('config')->where('id',1)->find();
        $model=explode(',',$config['search']);
       
        //搜索内容数量
        $count=Db::name('content')->where('status',1)->where('model','in',$model)->where('title','like','%'.$keyword.'%')->order('sort asc,id desc')->count();  

        // 获取标题，如果没有设置标题则使用名称
        $long['title']=$keyword.'-搜索结果'.'-'.$system['title'];
        $long['keywords']=$system['keywords'];
        $long['description']=$system['description'];
        $long['keyword']=$keyword;
      
        View::assign([
            'long'=>$long,
            'count'=>$count,
            'keyword'=>$keyword,
        ]);
    }

    // 留言
    public function msg()
    {

     
        //配置参数
        $config=Db::name('config')->where('id',1)->find();

        // 判断是否关闭留言功能
        if($config['message_status']=='0'){
            return json(['status' => 'fail', 'msg' => '系统已经关闭表单功能，请到后台开启再试！']);
        }

        // 获取提交的数据
        $data=input('post.');
        $form_id=input('fcode');
        $time=time();
        
        // 检查用户是否在 10 秒内重复提交
        $lastSubmitTime = Session::get('last_submit_time');
        if ($lastSubmitTime && ($time - $lastSubmitTime < 10)) {
            return json(['status' => 'fail', 'msg' => '您提交太频繁了，请稍后再试！']);
        }

        // 更新上次提交时间
        Session::set('last_submit_time', $time);

        // 获取用户 IP
        $ip = request()->ip();

        // 获取用户浏览器
        $userAgent = $_SERVER['HTTP_USER_AGENT'];
        $browser = $this->getBrowser($userAgent);

        // 获取操作系统信息
        $os = $this->getOS($userAgent);

        // 获取当前最大的 sign 值
        $maxSign = Db::name('msg')->max('sign');
        foreach($data as $k=>$v){
            $fcode=Db::name('fcode')->where('form_id',$form_id)->where('name',$k)->find();
            Db::name('msg')->strict(false)->insert(['sign'=>$maxSign + 1,'form_id'=>$form_id,'name'=>$fcode['description'],'field'=>$k,'value'=>$v,'time'=>$time,'ip'=>$ip,'browser'=>$browser,'os'=>$os]);
        }
        return json(['status'=>'success','msg'=>'留言成功！']);
        
    }

    /**
     * 根据设备类型渲染视图
     * 
     * @param string $template 模板名称
     * @param array $config 配置信息
     * @return mixed 渲染后的视图
     */
    private function renderForDevice($template, $config)
    {
        // 检查是否开启移动端并且当前是移动设备
        if ($config['open_wap'] == 1 && request()->isMobile()) {
            // 如果设置了移动端域名，则重定向
            if (!empty($config['wap_domain'])) {
                $wapUrl = preg_match('/^https?:\/\//i', $config['wap_domain']) ? 
                    $config['wap_domain'] : 'http://'.$config['wap_domain'];
                header("Location: $wapUrl");
                exit;
            }
            // 否则使用移动端模板
            return View::fetch('/wap/'.$template);
        }
        
        // 默认使用PC端模板
        return View::fetch($template);
    }

    /**
     * 从文件名中去除.html后缀
     * 
     * @param string $filename 文件名
     * @return string 处理后的文件名
     */
    private function cleanFilename($filename)
    {
        return str_replace('.html', '', $filename);
    }

    /**
     * 从请求中提取内容ID
     * 
     * @param Request $request 请求对象
     * @param string $url URL参数
     * @return string 内容ID
     */
    private function extractContentId($request, $url)
    {
        // 获取路径信息
        $pathinfo = $request->pathinfo();
        
        // 查找最后一个斜杠后面的内容
        $lastSegment = strrchr($pathinfo, '/');
        
        if ($lastSegment) {
            // 移除开头的斜杠
            $lastSegment = substr($lastSegment, 1);
        } else {
            // 如果没有斜杠，使用整个路径
            $lastSegment = $pathinfo;
        }
        
        // 去除 .html 后缀
        $contentId = str_replace('.html', '', $lastSegment);
        
        // 确保是有效的内容ID
        if (is_numeric($contentId) || preg_match('/^[a-zA-Z0-9\-_]+$/', $contentId)) {
            return $contentId;
        }
        
        // 回退方案：检查是否有明确的内容ID参数
        $params = $request->param();
        if (isset($params['id'])) {
            return $params['id'];
        }
        
        return '';
    }


    /**
     * 获取顶级栏目
     * @param int $categoryId 当前分类ID
     * @return array 顶级栏目数据
     */
    private function getTopCategory($categoryId)
    {
        $category=Db::name('category')->where('id',$categoryId)->find();
        while($category && $category['pid']!=0){
            $category=Db::name('category')->where('id',$category['pid'])->find();
        }
        return $category;
    }

    /**
     * 构建面包屑导航 - 支持无限级分类
     * @param int $categoryId 当前分类ID
     * @param string $contentName 内容标题(可选)
     * @return string 格式化的面包屑HTML
     */
    public function buildBreadcrumb($categoryId,$contentName='')
    {
        // 获取配置参数
        $config=Db::name('config')->where('id',1)->find();
        $symbol=$config['symbol']??' &gt; '; // 默认分隔符
        
        // 初始化面包屑数组，首页为第一个元素
        $breadcrumbs=[
            ['name'=>$config['indexname']??'首页','link'=>'/']
        ];
        
        // 递归获取所有父级分类
        $this->getParentCategories($categoryId,$breadcrumbs);
        
        
        // 构建HTML
        $html='';
        $count=count($breadcrumbs);
        
        foreach($breadcrumbs as $index=>$item){
            if(empty($item['link'])){
                $html.=$item['name'];
            }else{
                $html.='<a href="'.$item['link'].'">'.$item['name'].'</a>';
            }
            
            // 添加分隔符，最后一个元素后不添加
            if($index<$count-1){
                $html.=$symbol;
            }
        }
        
        return $html;
    }

    /**
     * 递归获取所有父级分类
     * @param int $categoryId 当前分类ID
     * @param array &$breadcrumbs 引用传递的面包屑数组
     */
    public function getParentCategories($categoryId,&$breadcrumbs)
    {
        // 获取当前分类信息
        $category=Db::name('category')->where('id',$categoryId)->find();
        if(!$category){
            return;
        }
        
        // 处理链接
        $link='';
        if(!empty($category['filename'])){
            $link='/'.$category['filename'].'/';
        }elseif(!empty($category['link'])){
            $link='/'.trim($category['link'],'/').'/';
        }
        
        // 如果有父级分类，先递归处理父级
        if($category['pid']!=0){
            $this->getParentCategories($category['pid'],$breadcrumbs);
        }
        
        // 将当前分类添加到面包屑数组
        $breadcrumbs[]=[
            'name'=>$category['name'],
            'link'=>$link
        ];
    }

    // 获取浏览器信息
    private function getBrowser($userAgent)
    {
        if (strpos($userAgent, 'MSIE') !== false || strpos($userAgent, 'Trident') !== false) {
            return 'Internet Explorer';
        } elseif (strpos($userAgent, 'Firefox') !== false) {
            return 'Firefox';
        } elseif (strpos($userAgent, 'Chrome') !== false) {
            return 'Chrome';
        } elseif (strpos($userAgent, 'Safari') !== false) {
            return 'Safari';
        } elseif (strpos($userAgent, 'Opera') !== false || strpos($userAgent, 'OPR') !== false) {
            return 'Opera';
        } else {
            return 'Other';
        }
    }

    // 获取操作系统信息
    private function getOS($userAgent)
    {
        if (preg_match('/Windows NT 10.0/i', $userAgent)) {
            return 'Windows 10';
        } elseif (preg_match('/Windows NT 6.3/i', $userAgent)) {
            return 'Windows 8.1';
        } elseif (preg_match('/Windows NT 6.2/i', $userAgent)) {
            return 'Windows 8';
        } elseif (preg_match('/Windows NT 6.1/i', $userAgent)) {
            return 'Windows 7';
        } elseif (preg_match('/Windows NT 6.0/i', $userAgent)) {
            return 'Windows Vista';
        } elseif (preg_match('/Windows NT 5.1/i', $userAgent)) {
            return 'Windows XP';
        } elseif (preg_match('/Mac OS X/i', $userAgent)) {
            return 'Mac OS X';
        } elseif (preg_match('/Linux/i', $userAgent)) {
            return 'Linux';
        } elseif (preg_match('/iPhone/i', $userAgent)) {
            return 'iPhone';
        } elseif (preg_match('/iPad/i', $userAgent)) {
            return 'iPad';
        } elseif (preg_match('/Android/i', $userAgent)) {
            return 'Android';
        } else {
            return 'Other';
        }
    }


}
