<?php
namespace app\index\controller;

use think\facade\View;
use think\Request;
use mysqli;
use think\facade\Db;

class Install
{
    public function index()
    {
        if(file_exists(__DIR__ . '/../../../install/install.lock')) {
            return redirect('/');
        }
        return View::fetch('install');
    }

    public function setup(Request $request)
    {
        $data = $request->post();

        // 数据库配置
        $servername = $data['db_host'];
        $username = $data['db_user'];
        $password = $data['db_pass'];
        $dbname = $data['db_name'];
        $hostport = $data['db_port'];

        // 创建数据库连接
        try {
            $db = new mysqli($servername, $username, $password);
            if ($db->connect_error) {
                throw new \Exception('数据库连接失败：' . $db->connect_error);
            }

            // 创建数据库
            $createDbQuery = "CREATE DATABASE IF NOT EXISTS `$dbname` CHARACTER SET utf8 COLLATE utf8_unicode_ci";
            if (!$db->query($createDbQuery)) {
                throw new \Exception('创建数据库失败：' . $db->error);
            }

            // 选择数据库
            $db->select_db($dbname);
        } catch (\Exception $e) {
            return '数据库连接或创建失败：' . $e->getMessage();
        }


        // 检查数据库是否有数据
        try {
            $result = $db->query("SHOW TABLES");
            if ($result->num_rows > 0) {
                // 清除数据库中的所有表
                while ($row = $result->fetch_array()) {
                    $table = $row[0];
                    $db->query("DROP TABLE IF EXISTS `$table`");
                }
            }
        } catch (\Exception $e) {
            return '清除旧数据失败：' . $e->getMessage();
        }


        // 动态更新数据库配置文件
        $dbConfigPath = __DIR__ . '/../../../config/database.php';
        $dbConfig = include $dbConfigPath;

        $dbConfig['connections']['mysql']['hostname'] = $servername;
        $dbConfig['connections']['mysql']['database'] = $dbname;
        $dbConfig['connections']['mysql']['username'] = $username;
        $dbConfig['connections']['mysql']['password'] = $password;
        $dbConfig['connections']['mysql']['hostport'] = $hostport;

        $dbConfigContent = "<?php\n\nreturn " . var_export($dbConfig, true) . ";\n";

        file_put_contents($dbConfigPath, $dbConfigContent);

        // 导入数据库文件
        try {
            $sql = file_get_contents(__DIR__ . '/../../../install/database.sql');
            $db->multi_query($sql);

            // 确保执行所有语句
            do {
                if ($result = $db->store_result()) {
                    $result->free();
                }
            } while ($db->more_results() && $db->next_result());
        } catch (\Exception $e) {
            return '导入数据库文件失败：' . $e->getMessage();
        }

        // 更新管理员账号密码
        try {
            $adminUser = $data['admin_user'];
            $adminPass = md5($data['admin_pass']); // 使用 md5 加密密码
            $updateQuery = "UPDATE admin SET adname='$adminUser', password='$adminPass' WHERE id=1";
            if (!$db->query($updateQuery)) {
                throw new \Exception('更新管理员账号密码失败：' . $db->error);
            }
        } catch (\Exception $e) {
            return '更新管理员账号密码失败：' . $e->getMessage();
        }

        // 标记安装完成
        try {
            file_put_contents(__DIR__ . '/../../../install/install.lock', 'installed');
        } catch (\Exception $e) {
            return '标记安装完成失败：' . $e->getMessage();
        }

        return View::fetch('install_success');
    }

    public function success()
    {
        return View::fetch('install_success');
    }
}